# -*- coding: utf-8 -*-

from qgis.core import ( qgsfunction, QgsSpatialIndex, QgsFeatureRequest,
    QgsMapLayerRegistry, NULL )

index_cache = {}

comuni_layer_name = "Com2014_WGS84_33N"
cod_istat_field_name = "PRO_COM"
ind_pu_layer_name = "Ind_pu"
ind_ln_layer_name = "Ind_ln"

def get_layer_index(layer):
    if layer.name() not in index_cache:
        index_cache[layer.name()] = QgsSpatialIndex(layer.getFeatures())
    return index_cache[layer.name()]


def cod_istat(geometry):
    comuni_layer = get_layer(comuni_layer_name)
    ind_pu_layer = get_layer(ind_pu_layer_name)

    intersectIds = get_layer_index(comuni_layer).intersects(geometry.boundingBox())

    features = comuni_layer.getFeatures(QgsFeatureRequest().setFilterFids(intersectIds))

    cod_istat = ""
    for f in features:
        if f.geometry().contains(geometry):
           if len(str(f[cod_istat_field_name])) == 5:
               cod_istat = "0" + str(f[cod_istat_field_name])
           else:
               cod_istat = str(f[cod_istat_field_name])
           return cod_istat,f["COMUNE"]

    return 0


def get_next_prog(layer, field_name):

    values = []
    idx = layer.fieldNameIndex(field_name)
    for f in layer.getFeatures():
        attrs = f.attributes()
        if attrs[idx] == NULL:
            continue
        if field_name == "ID_SPU" or field_name == "ID_SLN":
            values.append(int(attrs[idx][7:]))
        else:
            values.append(int(attrs[idx]))

    if not values:
        return '1'

    return str(max(values) + 1)


def get_layer(layer_name):
    layers = QgsMapLayerRegistry.instance().mapLayersByName(layer_name)
    if not layers:
        raise Exception( "Il layer" + layer_name + "non è stato trovato." )
    return layers[0]
